package it.unimi.dsi.lama4j;

/*		 
 * LaMa4J: Lattice Manipulation for Java
 *
 * Copyright (C) 2006-2016 Marco Rosa and Sebastiano Vigna 
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 2.1 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

public class Lattices {

	private Lattices() {}
	
	/** Output <a href="http://graphviz.org">Graphviz</a> code for the covering
	 * relation of a lattice.
	 * 
	 * <p>Note that the code will include just nodes and arcs. Initial and
	 * final code will have to be set up manually.
	 * 
	 * @param lattice a lattice.
	 * @param coveringRelation the covering relation of <code>lattice</code>,
	 * as computed by {@link Lattice#coveringRelation()}.
	 * @param writer a writer to output the code.
	 * @param labels whether labels should be added to the emitted code.
	 */
	
	public static void generateDot( Lattice lattice, Map<Element,Set<Element>> coveringRelation, PrintWriter writer, boolean labels ) {
		Collection<Element> elements = lattice.elements();
		Element element[] = elements.toArray( new Element[ elements.size() ] );
		Object2IntOpenHashMap<Element> element2pos = new Object2IntOpenHashMap<Element>(); 
		for( int i = 0; i < element.length; i++ ) {
			element2pos.put( element[ i ], i );
			System.out.print( "N" + i );
			if ( labels ) System.out.print( "[label=\"" + element[ i ] + "\"]" );
			System.out.println( ";" );
		}
		for( int i = 0; i < element.length; i++ ) {
			Set<Element> cover = coveringRelation.get( element[ i ] );
			if ( cover != null ) 
				for( Element t : cover ) 
					System.out.println( "N" + i + " -> " + "N" + element2pos.getInt( t ) + ";" );
		}
	}
}
