package it.unimi.dsi.lama4j;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

/*		 
 * LaMa4J: Lattice Manipulation for Java
 *
 * Copyright (C) 2006-2016 Marco Rosa and Sebastiano Vigna 
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 2.1 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


public class FreeTest extends LaMa4JTestCase {

	@Test
	public void testF3() {
		Lattice m = new Chain( 3, "x" );
		Lattice l = new Chain( 3, "y" );
		Lattice k = new Chain( 3, "z" );

		Sum f3 = (Sum)Sum.of( m, l, k  );
		assertLattice( f3 );
		SumTest.assertInjection( m, 0, f3 );
		SumTest.assertInjection( l, 1, f3 );
		SumTest.assertInjection( k, 2, f3 );
		
		assertEquals( 20, f3.elements().size() );
	}
	
	@Test
	public void testF3Equal() {
		Lattice l = new Chain( 3, "x" );

		Sum f3 = (Sum)Sum.of( l, l, l );
		assertLattice( f3 );
		SumTest.assertInjection( l, 0, f3 );
		SumTest.assertInjection( l, 1, f3 );
		SumTest.assertInjection( l, 2, f3 );
		
		assertEquals( 20, f3.elements().size() );
	}
	
	@Test
	public void testF4() {
		Lattice chain1 = new Chain( 3, "a" );
		Lattice chain2 = new Chain( 3, "b" );
		Lattice chain3 = new Chain( 3, "x" );
		Lattice chain4 = new Chain( 3, "y" );

		Sum f4 = (Sum)Sum.of( chain1, chain2, chain3, chain4 );
		SumTest.assertInjection( chain1, 0, f4 );
		SumTest.assertInjection( chain2, 1, f4 );
		SumTest.assertInjection( chain3, 2, f4 );
		SumTest.assertInjection( chain4, 3, f4 );

		// Too long. Util.verifyLattice( this, F4 );
		
		assertEquals( 168, f4.elements().size() );
	}

}
